<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
<!-- 
	Stylesheet for processing 2.1 output format test result files 
	To uses this directly in a browser, add the following to the JTL file as line 2:
	<? xml-stylesheet type="text/xsl" href="../extras/jmeter-results-report_21.xsl" ?>
	and you can then view the JTL in a browser
-->

	<xsl:output method="xml" />

	<xsl:template match="testResults">
			
		<jmeter>
			<xsl:call-template name="summary" />
		</jmeter>
		

	</xsl:template>



	<xsl:template name="summary">
			
		<xsl:variable name="allCount" select="count(/testResults/*)" />
		<xsl:variable name="allFailureCount" select="count(/testResults/*[attribute::s='false'])" />
		<xsl:variable name="allSuccessCount" select="count(/testResults/*[attribute::s='true'])" />
		<xsl:variable name="allSuccessPercent" select="$allSuccessCount div $allCount" />
		<xsl:variable name="allTotalTime" select="sum(/testResults/*/@t)" />
		<xsl:variable name="allAverageTime" select="$allTotalTime div $allCount" />
		<xsl:variable name="allMinTime">
				
			<xsl:for-each select="/testResults/*/@t">
				<xsl:sort data-type="number" order="ascending"/>
				<xsl:if test="position()=1">
					<xsl:value-of select="."/>
				</xsl:if>
			</xsl:for-each>
		</xsl:variable>
				
			
		<xsl:variable name="allMaxTime">
			<xsl:for-each select="/testResults/*/@t">
				<xsl:sort data-type="number" order="descending"/>
				<xsl:if test="position()=1">
					<xsl:value-of select="."/>
				</xsl:if>
			</xsl:for-each>
		</xsl:variable>
			
		<xsl:attribute name="class">
			<xsl:choose>
				<xsl:when test="$allFailureCount &gt; 0">Failure</xsl:when>
			</xsl:choose>
		</xsl:attribute>
		<allcount>
			<xsl:value-of select="$allCount" />
		</allcount>
		<allfailurecount>
			<xsl:value-of select="$allFailureCount" />
		</allfailurecount>
		<allsuccesspercent>
			<xsl:value-of select="$allSuccessPercent" />
			
		</allsuccesspercent>
		<allaveragetime>
			<xsl:value-of select="format-number($allAverageTime, 0)" />
			
		</allaveragetime>
		<allmintime>
			<xsl:value-of select="$allMinTime" />
			
		</allmintime>
		<allmaxtime>
			<xsl:value-of select="$allMaxTime" />
			
		</allmaxtime>

	</xsl:template>




	<xsl:template name="max">
		<xsl:param name="nodes" select="/.." />
		<xsl:choose>
			<xsl:when test="not($nodes)">NaN</xsl:when>
			<xsl:otherwise>
				<xsl:for-each select="$nodes">
					<xsl:sort data-type="number" order="descending" />
					<xsl:if test="position() = 1">
						<xsl:value-of select="number(.)" />
					</xsl:if>
				</xsl:for-each>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	

</xsl:stylesheet>