package net.sourceforge.cathcart.taskdefs;

import java.io.File;

import net.sourceforge.cathcart.typedefs.PropertyType;
import net.sourceforge.cathcart.typedefs.SizeComparison;

import org.apache.tools.ant.Project;
import org.apache.tools.ant.taskdefs.Copy;
import org.apache.tools.ant.taskdefs.Mkdir;
import org.apache.tools.ant.taskdefs.Move;
import org.apache.tools.ant.taskdefs.XSLTProcess;
import org.apache.tools.ant.taskdefs.XmlProperty;

/**
 * Task to check latest result against previous successful build..
 * 
 * @author Steven Hale
 * 
 */
public final class CheckPrevious extends CheckBase {

	private XSLTProcess xslt = new XSLTProcess();

	private Mkdir mkdir = new Mkdir();

	private XmlProperty xmlProperty = new XmlProperty();

	private Copy copy = new Copy();

	private Move move = new Move();
	
	public void execute() {
		parseConfig();
		fixPropertyTypes();
		outXml();
		setClasspath();

		performMkdir(mkdir);

		File out = new File(System.getProperty("java.io.tmpdir"), outXml);
		logger.log("Temporary file " + out.getAbsolutePath(), Project.MSG_VERBOSE);

		performXslt(xslt, out);
		performXmlProperty(xmlProperty, out, "");

		File previous = new File(storeDir, outXml);
		if (previous.exists()) {
			performXmlProperty(xmlProperty, previous, "previous");

			boolean improved = handleNestedElements();

			if (improved) {
				if (backup) {
					performMove(move, out);
				}
				performCopy(copy, out);
			}
		} else {
			logger.log("Result from previous build not found at " + storeDir + "/"
					+ outXml + ". No check made.", Project.MSG_WARN);
			performCopy(copy, out);
		}

	}

	private boolean handleNestedElements() {
		boolean improved = true;
		for (PropertyType propertyType : propertyTypes) {
			checkNestedElement(propertyType, false);
			String property = getProject().getProperty(
					tool + "." + propertyType.getName());

			logger.log(propertyType.getName() + "  " + property, Project.MSG_VERBOSE);

			String previousProperty = getProject().getProperty(
					"previous." + tool + "." + propertyType.getName());
			logger.log("previous." + tool + "." + propertyType.getName() + " "
					+ previousProperty, Project.MSG_VERBOSE);
			SizeComparison sizeComparison = chooseComparison(propertyType);

			if (previousProperty == null) {
				logger.log("Result from previous build not found for "
						+ propertyType.getName() + ". No check made.",
						Project.MSG_WARN);
			} else {
				setComparisonArguments(propertyType, property, new Double(
						previousProperty), sizeComparison);

				if (sizeComparison.eval()) {
					failure(propertyType, sizeComparison);

				} else {
					success(propertyType, sizeComparison);
					improved = shouldCopyToPrevious(improved, sizeComparison);

				}

			}

		}
		return improved;
	}

	public void outXml() {
		if (outXml == null) {
			outXml = getDefaultName() + ".xml";
		}
		logger.log("outXml " + outXml, Project.MSG_VERBOSE);

	}

	boolean shouldCopyToPrevious(boolean improved, SizeComparison sizeComparison) {
		sizeComparison.setLeeway(0);
		if (sizeComparison.eval()) {
			logger.log("No improvement: " + sizeComparison.toString(),
					Project.MSG_VERBOSE);
			improved = false;
		} else {
			logger.log("Improvement: " + sizeComparison.toString(),
					Project.MSG_VERBOSE);
		}
		return improved;
	}
	public void setXslt(XSLTProcess xslt) {
		this.xslt = xslt;
	}
	public void setMkdir(Mkdir mkdir) {
		this.mkdir = mkdir;
	}
	public void setXmlProperty(XmlProperty xmlProperty) {
		this.xmlProperty = xmlProperty;
	}
	public void setCopy(Copy copy) {
		this.copy = copy;
	}
	public void setMove(Move move) {
		this.move = move;
	}

}
